<?php

namespace Drp\Analytics\Foundation;

use Drp\Analytics\Contracts\HandlesAnalytics;
use Drp\Analytics\Foundation\Handlers\EloquentAnalytic;
use Drp\Analytics\Foundation\Handlers\EloquentMetric;
use Drp\Analytics\Jobs\StoreAnalytic;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;
use Illuminate\Support\Traits\Macroable;

class AnalyticsManager
{
    use Macroable;

    public const DRIVER_ELOQUENT = 'eloquent';

    public $handlers = [
        'analytic' => [
            self::DRIVER_ELOQUENT => EloquentAnalytic::class,
        ],
        'metric' => [
            self::DRIVER_ELOQUENT => EloquentMetric::class,
        ],
    ];

    public function store(string $type, $payload = null): void
    {
        $reference = new AnalyticReference(AnalyticableCookie::pull(), $type);
        $payload = new AnalyticPayload($payload);

        StoreAnalytic::dispatch($this->handlerFor($reference), $reference, $payload)
            ->onQueue(config('analytics.queue'));
    }

    public function get(string $type, ?callable $query = null): Collection
    {
        $concept = config("analytics.trackers.{$type}");
        $model = config("analytics.models.{$concept}");

        $baseQuery = $model::query()->whereType($type);

        if ($query === null) {
            return $baseQuery->get();
        }

        return $query($baseQuery)->get();
    }

    private function handlerFor(AnalyticReference $reference): HandlesAnalytics
    {
        $type = config("analytics.trackers.{$reference->getType()}");
        $handlerKey = config("analytics.drivers.{$type}");
        $handlerClass = Arr::get($this->handlers, "{$type}.{$handlerKey}");

        return new $handlerClass;
    }

    public function registerHandler(string $type, string $driver, string $handler): void
    {
        Arr::set($this->handlers, "{$type}.{$driver}", $handler);
    }

    public function renderJs(): string
    {
        return view('Analytics::analytics-js')->render();
    }
}
